@@ -3,11 +3,13 @@
 /// Bindings associate variables in the VM with constraints or values.
 use std::collections::{HashMap, HashSet};
 
-use crate::error::{PolarResult, RuntimeError};
+use crate::error::RuntimeError;
 use crate::folder::{fold_list, fold_term, Folder};
 use crate::terms::{has_rest_var, Operation, Operator, Symbol, Term, Value};
 use crate::vm::Goal;
 
+type Result<T> = core::result::Result<T, RuntimeError>;
+
 #[derive(Clone, Debug)]
 pub struct Binding(pub Symbol, pub Term);
 
@@ -157,12 +159,11 @@ impl BindingManager {
     ///
     /// If the binding succeeds, the new expression is returned as a goal. Otherwise,
     /// an error is returned.
-    fn partial_bind(&mut self, partial: Operation, var: &Symbol, val: Term) -> PolarResult<Goal> {
+    fn partial_bind(&mut self, partial: Operation, var: &Symbol, val: Term) -> Result<Goal> {
         match partial.ground(var, val.clone()) {
             None => Err(RuntimeError::IncompatibleBindings {
                 msg: "Grounding failed A".into(),
-            }
-            .into()),
+            }),
             Some(grounded) => {
                 self.add_binding(var, val);
                 Ok(Goal::Query {
@@ -200,7 +201,7 @@ impl BindingManager {
     ///
     /// If a binding between two variables is made, and one is bound and the other unbound, the
     /// unbound variable will take the value of the bound one.
-    pub fn bind(&mut self, var: &Symbol, val: Term) -> PolarResult<Option<Goal>> {
+    pub fn bind(&mut self, var: &Symbol, val: Term) -> Result<Option<Goal>> {
         let mut goal = None;
         if let Ok(symbol) = val.value().as_symbol() {
             goal = self.bind_variables(var, symbol)?;
@@ -215,8 +216,7 @@ impl BindingManager {
                 BindingManagerVariableState::Bound(_) => {
                     return Err(RuntimeError::IncompatibleBindings {
                         msg: format!("Cannot rebind {:?}", var),
-                    }
-                    .into())
+                    })
                 }
                 _ => self.add_binding(var, val.clone()),
             }
@@ -256,7 +256,7 @@ impl BindingManager {
     /// `term` must be an expression`.
     ///
     /// An error is returned if the constraint is incompatible with existing constraints.
-    pub fn add_constraint(&mut self, term: &Term) -> PolarResult<()> {
+    pub fn add_constraint(&mut self, term: &Term) -> Result<()> {
         self.do_followers(|_, follower| follower.add_constraint(term))?;
 
         assert!(term.value().as_expression().is_ok());
@@ -285,8 +285,7 @@ impl BindingManager {
                     None => {
                         return Err(RuntimeError::IncompatibleBindings {
                             msg: "Grounding failed B".into(),
-                        }
-                        .into())
+                        })
                     }
                 }
             }
@@ -429,7 +428,7 @@ impl BindingManager {
 // Private impls.
 impl BindingManager {
     /// Bind two variables together.
-    fn bind_variables(&mut self, left: &Symbol, right: &Symbol) -> PolarResult<Option<Goal>> {
+    fn bind_variables(&mut self, left: &Symbol, right: &Symbol) -> Result<Option<Goal>> {
         let mut goal = None;
         match (self._variable_state(left), self._variable_state(right)) {
             (
@@ -505,8 +504,7 @@ impl BindingManager {
             (BindingManagerVariableState::Bound(_), BindingManagerVariableState::Bound(_)) => {
                 return Err(RuntimeError::IncompatibleBindings {
                     msg: format!("{} and {} are both bound", left, right),
-                }
-                .into());
+                });
             }
             (
                 BindingManagerVariableState::Bound(left_value),
@@ -582,9 +580,9 @@ impl BindingManager {
         BindingManagerVariableState::Unbound
     }
 
-    fn do_followers<F>(&mut self, mut func: F) -> PolarResult<()>
+    fn do_followers<F>(&mut self, mut func: F) -> Result<()>
     where
-        F: FnMut(FollowerId, &mut BindingManager) -> PolarResult<()>,
+        F: FnMut(FollowerId, &mut BindingManager) -> Result<()>,
     {
         for (id, follower) in self.followers.iter_mut() {
             func(*id, follower)?
