@@ -14,6 +14,7 @@ mod debugger;
 pub mod diagnostic;
 pub mod error;
 pub mod events;
+pub mod filter;
 mod folder;
 pub mod formatting;
 mod inverter;
@@ -25,7 +26,7 @@ pub mod parser;
 mod partial;
 pub mod polar;
 pub mod query;
-mod resource_block;
+pub mod resource_block;
 mod rewrites;
 pub mod rules;
 mod runnable;
@@ -35,3 +36,4 @@ pub mod traces;
 mod validations;
 mod visitor;
 mod vm;
+pub mod warning;
