@@ -1,7 +1,10 @@
-use serde::{Deserialize, Serialize};
 use std::collections::VecDeque;
 use std::sync::{Arc, Mutex};
 
+use serde::{Deserialize, Serialize};
+
+use super::warning::PolarWarning;
+
 #[derive(Debug, Clone, Serialize, Deserialize)]
 pub enum MessageKind {
     Print,
@@ -15,10 +18,10 @@ pub struct Message {
 }
 
 impl Message {
-    pub fn warning(msg: String) -> Message {
+    pub fn warning(warning: PolarWarning) -> Message {
         Self {
             kind: MessageKind::Warning,
-            msg,
+            msg: warning.to_string(),
         }
     }
 }
