@@ -2,13 +2,15 @@ use std::collections::{HashMap, HashSet};
 use std::fmt;
 
 use crate::bindings::Bindings;
-use crate::error::{PolarResult, RuntimeError};
+use crate::error::RuntimeError;
 use crate::folder::{fold_term, Folder};
 use crate::formatting::ToPolarString;
 use crate::terms::*;
 
 use super::partial::{invert_operation, FALSE, TRUE};
 
+type Result<T> = core::result::Result<T, RuntimeError>;
+
 /// Set to `true` to debug performance in simplifier by turning on
 /// performance counters.
 const TRACK_PERF: bool = false;
@@ -137,7 +139,7 @@ pub fn simplify_bindings(bindings: Bindings) -> Option<Bindings> {
 ///
 /// - For partials, simplify the constraint expressions.
 /// - For non-partials, deep deref. TODO(ap/gj): deep deref.
-pub fn simplify_bindings_opt(bindings: Bindings, all: bool) -> PolarResult<Option<Bindings>> {
+pub fn simplify_bindings_opt(bindings: Bindings, all: bool) -> Result<Option<Bindings>> {
     let mut perf = PerfCounters::new(TRACK_PERF);
     simplify_debug!("simplify bindings");
 
@@ -198,10 +200,9 @@ pub fn simplify_bindings_opt(bindings: Bindings, all: bool) -> PolarResult<Optio
         } else if let Value::Expression(e) = value.value() {
             if e.variables().iter().all(|v| v.is_temporary_var()) {
                 return Err(RuntimeError::UnhandledPartial {
-                    term: value.clone(),
                     var: var.clone(),
-                }
-                .into());
+                    term: value.clone(),
+                });
             }
         }
     }
